# type: ignore[attr-defined]
"""Una coleccion de herramientas para facilitar el analisis y visualizacion de datos por @tacosdedatos."""

try:
    from importlib.metadata import version, PackageNotFoundError
except ImportError:  # pragma: no cover
    from importlib_metadata import version, PackageNotFoundError


try:
    __version__ = version(__name__)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"

from .manejo_de_archivos import *
from .manejo_de_datos import *
from .manejo_de_proyectos import *
from .viz import *



"""Utilidades para el manejo de proyectos."""

from pathlib import Path


def crear_proyecto(
    nombre: str = "nuevo-proyecto", tipo: str = "analisis", dir: str = "."
) -> None:
    """Crea una carpeta nueva para un poryecto con una estructura básica predeterminada. Como cookiecutter pero más básico.

    Parameters
    ----------
    nombre : str, optional
        Nombre de tu nuevo proyecto.
    tipo : str, optional
        Tipo de proyecto. Por el momento solo existe el proyecto de `analisis` de datos. Por default, "analisis".
    dir : str, optional
        Directorio donde crear el proyecto. Por default es en el que ejecutas el script `Path(.)`.
    """
    PROJECT_DIR = Path(dir).joinpath(nombre)

    DATA_DIR = PROJECT_DIR.joinpath("datos")
    DOCS_DIR = PROJECT_DIR.joinpath("docs")
    NOTEBOOKS_DIR = PROJECT_DIR.joinpath("notebooks")
    REPORTES_DIR = PROJECT_DIR.joinpath("reportes")
    SRC_DIR = PROJECT_DIR.joinpath("src")

    DATA_BRUTOS_DIR = DATA_DIR.joinpath("brutos")
    DATA_EXTERNOS_DIR = DATA_DIR.joinpath("externos")
    DATA_INTERINOS_DIR = DATA_DIR.joinpath("interinos")
    DATA_PROCESADOS_DIR = DATA_DIR.joinpath("procesados")
    DATA_FINALES_DIR = DATA_DIR.joinpath("finales")

    REPORTES_FIGURAS_DIR = REPORTES_DIR.joinpath("figuras")

    SRC_DATOS = SRC_DIR.joinpath("datos")
    SRC_EXTERNOS = SRC_DIR.joinpath("externos")
    SRC_HERRAMIENTAS = SRC_DIR.joinpath("herramientas")
    SRC_MODELOS = SRC_DIR.joinpath("modelos")
    SRC_VISUALIZACIONES = SRC_DIR.joinpath("visualizaciones")
    SRC_APPS = SRC_DIR.joinpath("apps")

    README = PROJECT_DIR.joinpath("README.md")
    AUTORES = PROJECT_DIR.joinpath("AUTORES.md")

    # Crea carpetas principales
    PROJECT_DIR.mkdir()
    DATA_DIR.mkdir()
    DOCS_DIR.mkdir()
    NOTEBOOKS_DIR.mkdir()
    REPORTES_DIR.mkdir()
    SRC_DIR.mkdir()

    # nivel 2
    DATA_BRUTOS_DIR.mkdir()
    DATA_EXTERNOS_DIR.mkdir()
    DATA_INTERINOS_DIR.mkdir()
    DATA_PROCESADOS_DIR.mkdir()
    DATA_FINALES_DIR.mkdir()

    REPORTES_FIGURAS_DIR.mkdir()

    SRC_DATOS.mkdir()
    SRC_EXTERNOS.mkdir()
    SRC_HERRAMIENTAS.mkdir()
    SRC_MODELOS.mkdir()
    SRC_VISUALIZACIONES.mkdir()
    SRC_APPS.mkdir()

    # archivos importantes
    README.touch()
    AUTORES.touch()

    DATA_BRUTOS_DIR.joinpath(".gitkeep").touch()
    DATA_EXTERNOS_DIR.joinpath(".gitkeep").touch()
    DATA_INTERINOS_DIR.joinpath(".gitkeep").touch()
    DATA_PROCESADOS_DIR.joinpath(".gitkeep").touch()
    DATA_FINALES_DIR.joinpath(".gitkeep").touch()

    REPORTES_FIGURAS_DIR.joinpath(".gitkeep").touch()

    SRC_DATOS.joinpath(".gitkeep").touch()
    SRC_EXTERNOS.joinpath(".gitkeep").touch()
    SRC_HERRAMIENTAS.joinpath(".gitkeep").touch()
    SRC_MODELOS.joinpath(".gitkeep").touch()
    SRC_VISUALIZACIONES.joinpath(".gitkeep").touch()
    SRC_APPS.joinpath(".gitkeep").touch()

    NOTEBOOKS_DIR.joinpath(".gitkeep").touch()
    DOCS_DIR.joinpath(".gitkeep").touch()



"""Utilidades para la visualización de datos"""
from pathlib import Path
from typing import List, Union

import defusedxml.ElementTree as ET


def tema_altair():
    """Un tema de Altair. Colores, dimensiones, tamaño de ticks, de etiquetas, de leyenda.

    Returns
    -------
    tema : Dict
        Un diccionario con las especificaciones del tema.
    """
    axisColor = "#282828"
    markColor = "#282828"
    backgroundColor = "#FFFAFA"
    font = "Helvetica"
    labelfont = "Helvetica"
    sourcefont = "Helvetica"
    gridColor = "#C9C9C9"
    axisLabelSize = 14
    return {
        "width": 600,
        "height": 400,
        "autosize": "pad",
        "config": {
            "padding": 10,
            "geoshape": {
                "fill": "#D9D9D6",
                "stroke": "#2B3E4B",
                "strokeWidth": 0.5,
            },
            "arc": {"fill": markColor,},
            "area": {"fill": markColor,},
            "axisBand": {"grid": False,},
            "axisBottom": {
                "domain": False,
                "domainColor": "black",
                "domainWidth": 3,
                "grid": True,
                "gridColor": gridColor,
                "gridWidth": 1,
                "labelFontSize": axisLabelSize,
                "labelFont": labelfont,
                "labelPadding": 8,
                "labelAngle": 0,
                "tickColor": axisColor,
                "tickSize": 10,
                "titleFontSize": 16,
                "titlePadding": 10,
                "titleFont": font,
                "title": "",
            },
            "axisLeft": {
                "domainColor": axisColor,
                "domainWidth": 1,
                "gridColor": gridColor,
                "gridWidth": 1,
                "labelFontSize": axisLabelSize,
                "labelFont": labelfont,
                "labelPadding": 8,
                "tickColor": axisColor,
                "tickSize": 10,
                "tickCount": 10,
                "ticks": True,
                "titleFontSize": 18,
                "titlePadding": 10,
                "titleFont": font,
                "titleAngle": 0,
                "titleX": -70,
                "titleY": -18,
                "titleAnchor": "start",
            },
            "axisRight": {
                "domainColor": axisColor,
                "domainWidth": 1,
                "gridColor": gridColor,
                "gridWidth": 1,
                "labelFontSize": axisLabelSize,
                "labelFont": labelfont,
                "labelPadding": 4,
                "tickColor": axisColor,
                "tickSize": 10,
                "ticks": True,
                "titleFontSize": 14,
                "titlePadding": 10,
                "titleFont": font,
            },
            "axisTop": {
                "domain": False,
                "domainColor": "black",
                "domainWidth": 3,
                "grid": True,
                "gridColor": gridColor,
                "gridWidth": 1,
                "labelFontSize": axisLabelSize,
                "labelFont": labelfont,
                "labelPadding": 4,
                "tickColor": axisColor,
                "tickSize": 10,
                "titleFontSize": 14,
                "titlePadding": 10,
                "titleFont": font,
            },
            "background": backgroundColor,
            "group": {"fill": backgroundColor,},
            "legend": {
                "labelFontSize": 18,
                "labelFont": labelfont,
                "labelLimit": 500,
                "padding": 5,
                "symbolSize": 200,
                "symbolType": "square",
                "titleFontSize": 20,
                "titlePadding": 5,
                "titleFont": font,
                "titleLimit": 400,
            },
            "line": {
                "color": markColor,
                "stroke": markColor,
                "strokeWidth": 3,
            },
            "rule": {
                "color": markColor,
                "stroke": markColor,
                "strokeWidth": 3,
            },
            "tick": {
                "color": markColor,
                "stroke": markColor,
                "strokeWidth": 1,
            },
            "trail": {
                "color": markColor,
                "stroke": markColor,
                "strokeWidth": 0,
                "size": 5,
            },
            "path": {"stroke": markColor, "strokeWidth": 0.5,},
            "point": {"filled": True,},
            "rect": {"fill": "#A20C4B", "opacity": 0.3,},
            "range": {
                "category": [
                    "#dc0d7a",
                    "#02a3cd",
                    "#e4a100",
                    "#dc0d12",
                    "#0DDC6F",
                    "#074a7e",
                    "#e46800",
                    "#aa3594",
                    "#a20c4b",
                ],
                "diverging": [
                    "#dc0d12",
                    "#e9686b",
                    "#fbe1e1",
                    "#dff4f9",
                    "#81d1e6",
                    "#03a3cd",
                ],
                "heatmap": [
                    "#fff7fd",
                    "#ffdbf7",
                    "#ffb9ec",
                    "#ff91dc",
                    "#ff67c7",
                    "#f83faf",
                    "#ef1d95",
                    "#e4007c",
                ],
            },
            "symbol": {
                "opacity": 1,
                "shape": "circle",
                "size": 40,
                "strokeWidth": 1,
            },
            "style": {
                "bar": {"binSpacing": 2, "fill": markColor, "stroke": False,},
                "text": {
                    "font": sourcefont,
                    "fontSize": 10,
                    "align": "right",
                    "fontWeight": 100,
                    "size": 10,
                },
            },
            "title": {
                "anchor": "start",
                "fontSize": 28,
                "fontWeight": 600,
                "font": font,
                "offset": 20,
            },
            "view": {"stroke": False, "padding": 15,},
            "header": {
                "fontWeight": 400,
                "labelFontSize": axisLabelSize,
                "labelFont": labelfont,
                "titleFontSize": 20,
                "titleFont": font,
                "title": " ",
                "titleBaseline": "bottom",
                "titleOffset": -30,
            },
        },
    }


def crear_paleta_tableau(
    colores: List[str], tipo: str, nombre: str, agregar_a_tableau: bool = False
) -> Union[str, None]:
    r"""Tranforma una lista de colores (en formato hex) en una paleta en formato XML para copiar y pegar en Preferences.tps para su uso en tableau. Si el argumento `agregar_a_tableau` es True puedes agregarla directamente con esta función.

    Parameters
    ----------
    colores : List[str]
        Una lista de colores en formato hex.
    tipo : str
        Tipo de paleta de colores: "divergente", "regular", o "secuencial".
    nombre : str
        Nombre de tu paleta de colores. Por ejemplo, "likert - rojo a azul"
    agregar_a_tableau : bool, optional
        Si es True, esta función busca tu `Preferences.tps` en `~/Documents/My Tableau Repository/` y agrega la paleta para que no tengas que hacerlo manualmente. Por default es False.

    Returns
    -------
    Union[str, None]
        La paleta lista en el formato correcto para `Preferences.tps`. Si `agregar_a_tableau` es True, no se regresa la paleta. Se agrega directo a Preferences.tps

    
    Examples
    --------
    `crear_paleta_tableau()` te da una cadena de caracteres con caracteres especiales (como el \n para saltar líneas o \t para tabs). Asegurate de hacer `print()` el resultado de la función para copiar y pegar en `Preferences.tps`.
    Por ejemplo:

    >>> import tacosdedatos_utils as tdd 
    >>> colores_tdd = ['#dc0d7a', '#02a3cd', '#e4a100', '#dc0d12', '#0DDC6F', '#074a7e', '#e46800', '#aa3594', '#a20c4b',] 
    >>> paleta_nueva = tdd.crear_paleta_tableau(colores = colores_tdd, tipo = 'regular', nombre = 'tacosdedatos - colores principales')
    >>> print(paleta_nueva)
    <color-palette name="tacosdedatos - colores principales" type="regular">
       <color>#dc0d7a</color>
       <color>#02a3cd</color>
       <color>#e4a100</color>
       <color>#dc0d12</color>
       <color>#0DDC6F</color>
       <color>#074a7e</color>
       <color>#e46800</color>
       <color>#aa3594</color>
       <color>#a20c4b</color>
    </color-palette>
    """
    tipos_de_paletas = ["regular", "secuencial", "divergente"]
    if tipo not in tipos_de_paletas:
        raise TypeError(
            f"Las paletas pueden ser solo `regular`, `secuencial`, or `divergente`, tu escogiste: {tipo}"
        )

    if tipo == "secuencial":
        tipo = "ordered-sequential"
    if tipo == "divergente":
        tipo = "ordered-divergent"

    lista_de_colores = [f"\t<color>{color}</color>\n" for color in colores]
    colores_de_la_paleta = "".join(lista_de_colores)

    paleta = f'<color-palette name="{nombre}" type="{tipo}">\n{colores_de_la_paleta}</color-palette>\n'

    if agregar_a_tableau:
        repo_tableau = Path.home() / "Documents/My Tableau Repository/"
        if repo_tableau.exists():
            arbol = ET.parse(repo_tableau / "Preferences.tps")
            raiz = arbol.getroot()
            preferences = raiz[0]
            nueva_paleta = ET.fromstring(paleta)
            preferences.append(nueva_paleta)
            arbol.write(repo_tableau / "Preferences.tps")
        else:
            raise OSError(
                f"No encontramos la carpeta {str(repo_tableau)} asegurate que exista y tengas acceso a ella."
            )
        return None
    else:
        return paleta



"""Utilidades para el manejo de datos."""

from typing import Union

import pandas as pd
from IPython.display import display_html


def mostrar_dfs(
    *dfs: Union[pd.DataFrame, pd.Series], espacio: int = 20
) -> None:
    """Muestra más de un DataFrame o Serie de pandas en una celda de jupyter notebooks.

    Parameters
    ----------
    dfs : Union[pd.DataFrame, pd.Series]
        Los objetos (DataFrame o Series de pandas) que quieres mostrar.
    espacio : int, optional
        El espacio entre cada DataFrame en pixeles, por default 20 
    """
    html = f"""<div style="display:flex;">"""
    for df in dfs:
        try:
            html_del_df = df._repr_html_()
        except AttributeError:
            html_del_df = f"<pre>{repr(df)}</pre>"

        html += f"<div style='margin-right:{espacio}px;'>{html_del_df}</div>"

    html += "</div>"
    display_html(html, raw=True)



"""Utilidades para el manejo de archivos."""

from pathlib import Path


def arbol(carpeta):
    """
    Muestra el árbol de una carpeta.
    """
    carpeta = Path(carpeta)
    print(f"+ {carpeta}")
    for ruta in sorted(carpeta.rglob("[!.]*")):
        profundidad = len(ruta.relative_to(carpeta).parts)
        espacio = "    " * profundidad
        print(f"{espacio}+ {ruta.name}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tacosdedatos_utils']

package_data = \
{'': ['*']}

install_requires = \
['defusedxml>=0.6.0,<0.7.0',
 'ipython>=7.16.1,<8.0.0',
 'pandas>=1.0.0,<2.0.0',
 'rich>=2.2.2,<3.0.0',
 'typer[all]>=0.2.1,<0.3.0']

extras_require = \
{':python_version < "3.8"': ['importlib_metadata>=1.6.0,<2.0.0']}

entry_points = \
{'console_scripts': ['tacosdedatos-utils = tacosdedatos_utils.__main__:app']}

setup_kwargs = {
    'name': 'tacosdedatos-utils',
    'version': '1.3.0',
    'description': 'Una coleccion de herramientas para facilitar el analisis y visualizacion de datos por @tacosdedatos.',
    'long_description': '# Bienvenidx a la documentación de `tacosdedatos-utils`\n\nUna colección de herramientas para facilitar el análisis y visualización de datos por [@tacosdedatos](https://twitter.com/tacosdedatos).\n\n\n<div align="center">\n\n[![PyPI version](https://badge.fury.io/py/tacosdedatos-utils.svg)](https://badge.fury.io/py/tacosdedatos-utils)\n[![Documentation Status](https://readthedocs.org/projects/tacosdedatos-utils/badge/?version=latest)](https://tacosdedatos-utils.readthedocs.io/es/latest/?badge=latest)\n[![Build status](https://github.com/tacosdedatos/tacosdedatos-utils/workflows/build/badge.svg?branch=master&event=push)](https://github.com/tacosdedatos/tacosdedatos-utils/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/tacosdedatos-utils.svg)](https://pypi.org/project/tacosdedatos-utils/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/tacosdedatos/tacosdedatos-utils/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n[![Descargas mensuales](https://img.shields.io/pypi/dm/tacosdedatos-utils?color=%23dc0d7a&label=descargas)](https://img.shields.io/pypi/dm/tacosdedatos-utils?color=%23dc0d7a&label=descargas)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/tacosdedatos/tacosdedatos-utils/blob/master/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%F0%9F%9A%80-semantic%20versions-informational.svg)](https://github.com/tacosdedatos/tacosdedatos-utils/releases)\n[![License](https://img.shields.io/github/license/tacosdedatos/tacosdedatos-utils)](https://github.com/tacosdedatos/tacosdedatos-utils/blob/master/LICENSE)\n\n</div>\n\n## Que trae\n\n* la función `arbol` para mostrar los contenidos de la carpeta que le pases.\n```python\nimport tacosdedatos_utils as tdd\n\ntdd.arbol("./notebooks")\n>>>> + notebooks\n        + Cpp.ipynb\n        + Data.ipynb\n        + Fasta.ipynb\n        + Lorenz.ipynb\n        + R.ipynb\n        + audio\n            + audio.wav\n        + bqplot.ipynb\n        + images\n            + marie.png\n            + xeus-cling.png\n            + xtensor.png\n            + xwidgets.png\n        + lorenz.py\n        + pandas.ipynb\n```\n\n* la función `crear_proyecto`\n```python\nimport tacosdedatos_utils as tdd \n\ntdd.crear_proyecto(nombre = "proyecto-de-analisis-de-datos")\n\ntdd.arbol("proyecto-de-analisis-de-datos/")\n>>>> + proyecto-de-analisis-de-datos\n        + AUTORES.md\n        + README.md\n        + datos\n            + brutos\n            + externos\n            + finales\n            + interinos\n            + procesados\n        + docs\n        + notebooks\n        + reportes\n            + figuras\n        + src\n            + apps\n            + datos\n            + externos\n            + herramientas\n            + modelos\n            + visualizaciones\n```\nque también funciona desde tu línea de comandos.\n\n![GIF mostrando como usar la linea de comandos con tacosdedatos-utils](https://github.com/chekos/pics_for_github/blob/master/2020-06-17%2014.06.39.gif?raw=true)\n\n\n## 📃 Citeishon\n\n```\n@misc{tacosdedatos-utils,\n  author = {tacosdedatos},\n  title = {Una coleccion de herramientas para facilitar el analisis y visualizacion de datos por @tacosdedatos.},\n  year = {2020},\n  publisher = {GitHub},\n  journal = {GitHub repository},\n  howpublished = {\\url{https://github.com/tacosdedatos/tacosdedatos-utils}}\n}\n```\n\n> Este proyecto fue generado con [`python-package-template`](https://github.com/TezRomacH/python-package-template).\n',
    'author': 'tacosdedatos',
    'author_email': 'chekos@tacosdedatos.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tacosdedatos/tacosdedatos-utils',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
